@@ -1466,21 +1466,30 @@ void action_move_relative(union ActionData *data)
 void action_resize_relative(union ActionData *data)
 {
     ObClient *c = data->relative.any.c;
-    gint x, y, ow, w, oh, h, lw, lh;
+    gint x, y, ow, xoff, nw, oh, yoff, nh, lw, lh;
 
     client_action_start(data);
 
     x = c->area.x;
     y = c->area.y;
     ow = c->area.width;
-    w = ow + data->relative.deltax * c->size_inc.width
+    xoff = -data->relative.deltaxl * c->size_inc.width;
+    nw = ow + data->relative.deltax * c->size_inc.width
         + data->relative.deltaxl * c->size_inc.width;
     oh = c->area.height;
-    h = oh + data->relative.deltay * c->size_inc.height
+    yoff = -data->relative.deltayu * c->size_inc.height;
+    nh = oh + data->relative.deltay * c->size_inc.height
         + data->relative.deltayu * c->size_inc.height;
+
+    g_print("deltax %d %d x %d ow %d xoff %d nw %d\n",
+            data->relative.deltax, 
+            data->relative.deltaxl, 
+            x, ow, xoff, nw);
     
-    client_try_configure(c, &x, &y, &w, &h, &lw, &lh, TRUE);
-    client_move_resize(c, x + (ow - w), y + (oh - h), w, h);
+    client_try_configure(c, &x, &y, &nw, &nh, &lw, &lh, TRUE);
+    xoff = xoff == 0 ? 0 : (xoff < 0 ? MAX(xoff, ow-nw) : MIN(xoff, ow-nw));
+    yoff = yoff == 0 ? 0 : (yoff < 0 ? MAX(yoff, oh-nh) : MIN(yoff, oh-nh));
+    client_move_resize(c, x + xoff, y + yoff, nw, nh);
     client_action_end(data, FALSE);
 }
 
