@@ -2,7 +2,7 @@
 
    client_list_menu.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,20 +38,22 @@ typedef struct
     guint desktop;
 } DesktopData;
 
-static void desk_menu_update(ObMenuFrame *frame, gpointer data)
+static gboolean desk_menu_update(ObMenuFrame *frame, gpointer data)
 {
     ObMenu *menu = frame->menu;
     DesktopData *d = data;
     GList *it;
     gint i;
-    gboolean icons = FALSE;
     gboolean empty = TRUE;
+    gboolean onlyiconic = TRUE;
 
     menu_clear_entries(menu);
 
-    for (it = focus_order[d->desktop], i = 0; it; it = g_list_next(it), ++i) {
+    for (it = focus_order, i = 0; it; it = g_list_next(it), ++i) {
         ObClient *c = it->data;
-        if (client_normal(c) && !c->skip_taskbar) {
+        if (client_normal(c) && (!c->skip_taskbar || c->iconic) &&
+            (c->desktop == d->desktop || c->desktop == DESKTOP_ALL))
+        {
             GSList *acts = NULL;
             ObAction* act;
             ObMenuEntry *e;
@@ -59,11 +61,6 @@ static void desk_menu_update(ObMenuFrame *frame, gpointer data)
 
             empty = FALSE;
 
-            if (!icons && c->iconic) {
-                icons = TRUE;
-                menu_add_separator(menu, -1);
-            }
-
             act = action_from_string("Activate",
                                      OB_USER_ACTION_MENU_SELECTION);
             act->data.activate.any.c = c;
@@ -72,19 +69,31 @@ static void desk_menu_update(ObMenuFrame *frame, gpointer data)
                                      OB_USER_ACTION_MENU_SELECTION);
             act->data.desktop.desk = d->desktop;
             acts = g_slist_append(acts, act);
-            e = menu_add_normal(menu, i,
-                                (c->iconic ? c->icon_title : c->title), acts);
 
-            if (config_menu_client_list_icons && (icon = client_icon(c, 32, 32))) {
+            if (c->iconic) {
+                gchar *title = g_strdup_printf("(%s)", c->icon_title);
+                e = menu_add_normal(menu, i, title, acts, FALSE);
+                g_free(title);
+            } else {
+                onlyiconic = FALSE;
+                e = menu_add_normal(menu, i, c->title, acts, FALSE);
+            }
+
+            if (config_menu_client_list_icons
+                && (icon = client_icon(c, 32, 32))) {
                 e->data.normal.icon_width = icon->width;
                 e->data.normal.icon_height = icon->height;
                 e->data.normal.icon_data = icon->data;
+                e->data.normal.icon_alpha = c->iconic ? OB_ICONIC_ALPHA : 0xff;
             }
         }
     }
 
-    if (empty) {
-        /* no entries */
+    if (empty || onlyiconic) {
+        /* no entries or only iconified windows, so add a
+         * way to go to this desktop without uniconifying a window */
+        if (!empty)
+            menu_add_separator(menu, -1, NULL);
 
         GSList *acts = NULL;
         ObAction* act;
@@ -93,21 +102,24 @@ static void desk_menu_update(ObMenuFrame *frame, gpointer data)
         act = action_from_string("Desktop", OB_USER_ACTION_MENU_SELECTION);
         act->data.desktop.desk = d->desktop;
         acts = g_slist_append(acts, act);
-        e = menu_add_normal(menu, 0, _("Go there..."), acts);
+        e = menu_add_normal(menu, 0, _("Go there..."), acts, TRUE);
         if (d->desktop == screen_desktop)
             e->data.normal.enabled = FALSE;
     }
+    return TRUE; /* always show */
 }
 
 /* executes it using the client in the actions, since we set that
    when we make the actions! */
-static void desk_menu_execute(ObMenuEntry *self, guint state, gpointer data)
+static void desk_menu_execute(ObMenuEntry *self, ObMenuFrame *f,
+                              ObClient *c, guint state, gpointer data,
+                              Time time)
 {
     ObAction *a;
 
     if (self->data.normal.actions) {
         a = self->data.normal.actions->data;
-        action_run(self->data.normal.actions, a->data.any.c, state);
+        action_run(self->data.normal.actions, a->data.any.c, state, time);
     }
 }
 
@@ -120,39 +132,37 @@ static void desk_menu_destroy(ObMenu *menu, gpointer data)
     desktop_menus = g_slist_remove(desktop_menus, menu);
 }
 
-static void self_update(ObMenuFrame *frame, gpointer data)
+static gboolean self_update(ObMenuFrame *frame, gpointer data)
 {
     ObMenu *menu = frame->menu;
     guint i;
-    GSList *it, *next;
+
+    menu_clear_entries(menu);
+
+    while (desktop_menus) {
+        menu_free(desktop_menus->data);
+        desktop_menus = g_slist_delete_link(desktop_menus, desktop_menus);
+    }
     
-    it = desktop_menus;
     for (i = 0; i < screen_num_desktops; ++i) {
-        if (!it) {
-            ObMenu *submenu;
-            gchar *name = g_strdup_printf("%s-%u", MENU_NAME, i);
-            DesktopData *data = g_new(DesktopData, 1);
+        ObMenu *submenu;
+        gchar *name = g_strdup_printf("%s-%u", MENU_NAME, i);
+        DesktopData *data = g_new(DesktopData, 1);
 
-            data->desktop = i;
-            submenu = menu_new(name, screen_desktop_names[i], data);
-            menu_set_update_func(submenu, desk_menu_update);
-            menu_set_execute_func(submenu, desk_menu_execute);
-            menu_set_destroy_func(submenu, desk_menu_destroy);
+        data->desktop = i;
+        submenu = menu_new(name, screen_desktop_names[i], FALSE, data);
+        menu_set_update_func(submenu, desk_menu_update);
+        menu_set_execute_func(submenu, desk_menu_execute);
+        menu_set_destroy_func(submenu, desk_menu_destroy);
 
-            menu_add_submenu(menu, i, name);
+        menu_add_submenu(menu, i, name);
 
-            g_free(name);
+        g_free(name);
 
-            desktop_menus = g_slist_append(desktop_menus, submenu);
-        } else
-            it = g_slist_next(it);
-    }
-    for (; it; it = next, ++i) {
-        next = g_slist_next(it);
-        menu_free(it->data);
-        desktop_menus = g_slist_delete_link(desktop_menus, it);
-        menu_entry_remove(menu_find_entry_id(menu, i));
+        desktop_menus = g_slist_append(desktop_menus, submenu);
     }
+
+    return TRUE; /* always show */
 }
 
 static void client_dest(ObClient *client, gpointer data)
@@ -181,14 +191,14 @@ void client_list_menu_startup(gboolean reconfig)
     ObMenu *menu;
 
     if (!reconfig)
-        client_add_destructor(client_dest, NULL);
+        client_add_destroy_notify(client_dest, NULL);
 
-    menu = menu_new(MENU_NAME, _("Desktops"), NULL);
+    menu = menu_new(MENU_NAME, _("Desktops"), TRUE, NULL);
     menu_set_update_func(menu, self_update);
 }
 
 void client_list_menu_shutdown(gboolean reconfig)
 {
     if (!reconfig)
-        client_remove_destructor(client_dest);
+        client_remove_destroy_notify(client_dest);
 }
