@@ -2,7 +2,7 @@
 
    focus.h for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,15 +29,9 @@ struct _ObClient;
 
 /*! The client which is currently focused */
 extern struct _ObClient *focus_client;
-/*! The client which is being decorated as focused, not always matching the
-  real focus, but this is used to track it so that it can be ersolved to match
-*/
-extern struct _ObClient *focus_hilite;
-/*! The client which appears focused during a focus cycle operation */
-extern struct _ObClient *focus_cycle_target;
 
 /*! The recent focus order on each desktop */
-extern GList **focus_order;
+extern GList *focus_order;
 
 void focus_startup(gboolean reconfig);
 void focus_shutdown(gboolean reconfig);
@@ -46,24 +40,13 @@ void focus_shutdown(gboolean reconfig);
   send focus anywhere, its called by the Focus event handlers */
 void focus_set_client(struct _ObClient *client);
 
-typedef enum {
-    OB_FOCUS_FALLBACK_UNFOCUSING, /*!< forcefully remove focus from the
-                                    current window */
-    OB_FOCUS_FALLBACK_CLOSED,     /*!< closed the window with focus */
-    OB_FOCUS_FALLBACK_NOFOCUS     /*!< nothing has focus for some reason */
-} ObFocusFallbackType;
-
-struct _ObClient* focus_fallback_target(ObFocusFallbackType type);
+/*! Focus nothing, but let keyboard events be caught. */
+void focus_nothing();
 
 /*! Call this when you need to focus something! */
-void focus_fallback(ObFocusFallbackType type);
-
-/*! Cycle focus amongst windows. */
-void focus_cycle(gboolean forward, gboolean linear, gboolean interactive,
-                 gboolean dialog, gboolean done, gboolean cancel);
-void focus_directional_cycle(ObDirection dir, gboolean interactive,
-                             gboolean dialog, gboolean done, gboolean cancel);
-void focus_cycle_draw_indicator();
+struct _ObClient* focus_fallback(gboolean allow_refocus,
+                                 gboolean allow_pointer,
+                                 gboolean allow_omnipresent);
 
 /*! Add a new client into the focus order */
 void focus_order_add_new(struct _ObClient *c);
@@ -78,4 +61,6 @@ void focus_order_to_top(struct _ObClient *c);
   very bottom always though). */
 void focus_order_to_bottom(struct _ObClient *c);
 
+struct _ObClient *focus_order_find_first(guint desktop);
+
 #endif
