@@ -2,7 +2,7 @@
 
    frame.h for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,17 +27,23 @@ typedef struct _ObFrame ObFrame;
 
 struct _ObClient;
 
+typedef void (*ObFrameIconifyAnimateFunc)(gpointer data);
+
 typedef enum {
     OB_FRAME_CONTEXT_NONE,
     OB_FRAME_CONTEXT_DESKTOP,
+    OB_FRAME_CONTEXT_ROOT,
     OB_FRAME_CONTEXT_CLIENT,
     OB_FRAME_CONTEXT_TITLEBAR,
-    OB_FRAME_CONTEXT_HANDLE,
     OB_FRAME_CONTEXT_FRAME,
     OB_FRAME_CONTEXT_BLCORNER,
     OB_FRAME_CONTEXT_BRCORNER,
     OB_FRAME_CONTEXT_TLCORNER,
     OB_FRAME_CONTEXT_TRCORNER,
+    OB_FRAME_CONTEXT_TOP,
+    OB_FRAME_CONTEXT_BOTTOM,
+    OB_FRAME_CONTEXT_LEFT,
+    OB_FRAME_CONTEXT_RIGHT,
     OB_FRAME_CONTEXT_MAXIMIZE,
     OB_FRAME_CONTEXT_ALLDESKTOPS,
     OB_FRAME_CONTEXT_SHADE,
@@ -71,17 +77,13 @@ struct _ObFrame
     struct _ObClient *client;
 
     Window    window;
-    Window    plate;
 
     Strut     size;
     Rect      area;
     gboolean  visible;
 
-    /*! Whether the window is obscured at all or fully visible. */
-    gboolean obscured;
-
+    guint     functions;
     guint     decorations;
-    gboolean  max_horz;
 
     Window    title;
     Window    label;
@@ -95,8 +97,41 @@ struct _ObFrame
     Window    lgrip;
     Window    rgrip;
 
-    Window    tlresize;
-    Window    trresize;
+    /* These are borders of the frame and its elements */
+    Window    titleleft;
+    Window    titletop;
+    Window    titletopleft;
+    Window    titletopright;
+    Window    titleright;
+    Window    titlebottom;
+    Window    left;
+    Window    right;
+    Window    handleleft;
+    Window    handletop;
+    Window    handleright;
+    Window    handlebottom;
+    Window    lgriptop;
+    Window    lgripleft;
+    Window    lgripbottom;
+    Window    rgriptop;
+    Window    rgripright;
+    Window    rgripbottom;
+    Window    innerleft;    /*!< For drawing the inner client border */
+    Window    innertop;     /*!< For drawing the inner client border */
+    Window    innerright;   /*!< For drawing the inner client border */
+    Window    innerbottom;  /*!< For drawing the inner client border */
+    Window    backback;     /*!< A colored window shown while resizing */
+    Window    backfront;    /*!< An undrawn-in window, to prevent flashing on
+                                 unmap */
+
+    /* These are resize handles inside the titlebar */
+    Window    topresize;
+    Window    tltresize;
+    Window    tllresize;
+    Window    trtresize;
+    Window    trrresize;
+
+    Colormap  colormap;
 
     RrAppearance *a_unfocused_title;
     RrAppearance *a_focused_title;
@@ -106,12 +141,18 @@ struct _ObFrame
     RrAppearance *a_unfocused_handle;
     RrAppearance *a_focused_handle;
 
-    Strut     innersize;
-
     GSList   *clients;
 
-    gint      width;         /* title and handle */
-    gint      label_width;
+    gint      icon_on;    /* if the window icon button is on */
+    gint      label_on;   /* if the window title is on */
+    gint      iconify_on; /* if the window iconify button is on */
+    gint      desk_on;    /* if the window all-desktops button is on */
+    gint      shade_on;   /* if the window shade button is on */
+    gint      max_on;     /* if the window maximize button is on */
+    gint      close_on;   /* if the window close button is on */
+
+    gint      width;         /* width of the titlebar and handle */
+    gint      label_width;   /* width of the label in the titlebar */
     gint      icon_x;        /* x-position of the window icon button */
     gint      label_x;       /* x-position of the window title */
     gint      iconify_x;     /* x-position of the window iconify button */
@@ -120,9 +161,16 @@ struct _ObFrame
     gint      max_x;         /* x-position of the window maximize button */
     gint      close_x;       /* x-position of the window close button */
     gint      bwidth;        /* border width */
-    gint      rbwidth;       /* title border width */
-    gint      cbwidth_x;     /* client border width */
-    gint      cbwidth_y;     /* client border width */
+    gint      cbwidth_l;     /* client border width */
+    gint      cbwidth_t;     /* client border width */
+    gint      cbwidth_r;     /* client border width */
+    gint      cbwidth_b;     /* client border width */
+    gboolean  max_horz;      /* when maxed some decorations are hidden */
+    gboolean  max_vert;      /* when maxed some decorations are hidden */
+
+    /* the leftmost and rightmost elements in the titlebar */
+    ObFrameContext leftmost;
+    ObFrameContext rightmost;
 
     gboolean  max_press;
     gboolean  close_press;
@@ -140,39 +188,58 @@ struct _ObFrame
     gboolean  flashing;
     gboolean  flash_on;
     GTimeVal  flash_end;
+
+    /*! Is the frame currently in an animation for iconify or restore.
+      0 means that it is not animating. > 0 means it is animating an iconify.
+      < 0 means it is animating a restore.
+    */
+    gint iconify_animation_going;
+    GTimeVal  iconify_animation_end;
 };
 
-ObFrame *frame_new();
+ObFrame *frame_new(struct _ObClient *c);
+void frame_free(ObFrame *self);
+
 void frame_show(ObFrame *self);
 void frame_hide(ObFrame *self);
 void frame_adjust_theme(ObFrame *self);
 void frame_adjust_shape(ObFrame *self);
 void frame_adjust_area(ObFrame *self, gboolean moved,
                        gboolean resized, gboolean fake);
+void frame_adjust_client_area(ObFrame *self);
 void frame_adjust_state(ObFrame *self);
 void frame_adjust_focus(ObFrame *self, gboolean hilite);
 void frame_adjust_title(ObFrame *self);
 void frame_adjust_icon(ObFrame *self);
-void frame_grab_client(ObFrame *self, struct _ObClient *client);
-void frame_release_client(ObFrame *self, struct _ObClient *client);
+void frame_grab_client(ObFrame *self);
+void frame_release_client(ObFrame *self);
 
 ObFrameContext frame_context_from_string(const gchar *name);
 
-ObFrameContext frame_context(struct _ObClient *self, Window win);
+ObFrameContext frame_context(struct _ObClient *self, Window win,
+                             gint x, gint y);
 
 /*! Applies gravity to the client's position to find where the frame should
   be positioned.
   @return The proper coordinates for the frame, based on the client.
 */
-void frame_client_gravity(ObFrame *self, gint *x, gint *y);
+void frame_client_gravity(ObFrame *self, gint *x, gint *y, gint w, gint h);
 
 /*! Reversly applies gravity to the frame's position to find where the client
   should be positioned.
     @return The proper coordinates for the client, based on the frame.
 */
-void frame_frame_gravity(ObFrame *self, gint *x, gint *y);
+void frame_frame_gravity(ObFrame *self, gint *x, gint *y, gint w, gint h);
 
 void frame_flash_start(ObFrame *self);
 void frame_flash_stop(ObFrame *self);
 
+/*! Start an animation for iconifying or restoring a frame. The callback
+  will be called when the animation finishes. But if another animation is
+  started in the meantime, the callback will never get called. */
+void frame_begin_iconify_animation(ObFrame *self, gboolean iconifying);
+void frame_end_iconify_animation(ObFrame *self);
+
+#define frame_iconify_animating(f) (f->iconify_animation_going != 0)
+
 #endif
