@@ -87,6 +87,7 @@ static void set_curpos(KeyBindingTree *newpos)
     if (curpos != NULL) {
         gchar *text = NULL;
         GList *it;
+        Rect *a;
 
         for (it = curpos->keylist; it; it = g_list_next(it)) {
             gchar *oldtext = text;
@@ -97,7 +98,8 @@ static void set_curpos(KeyBindingTree *newpos)
             g_free(oldtext);
         }
 
-        popup_position(popup, NorthWestGravity, 10, 10);
+        a = screen_physical_area_monitor_active();
+        popup_position(popup, NorthWestGravity, a->x + 10, a->y + 10);
         /* 1 second delay for the popup to show */
         popup_delay_show(popup, G_USEC_PER_SEC, text);
         g_free(text);
@@ -143,7 +145,6 @@ gboolean keyboard_bind(GList *keylist, ObAction *action)
 {
     KeyBindingTree *tree, *t;
     gboolean conflict;
-    gboolean mods = TRUE;
 
     g_assert(keylist != NULL);
     g_assert(action != NULL);
@@ -164,17 +165,13 @@ gboolean keyboard_bind(GList *keylist, ObAction *action)
         return FALSE;
     }
 
-    /* find if every key in this chain has modifiers, and also find the
-       bottom node of the tree */
-    while (t->first_child) {
-        if (!t->state)
-            mods = FALSE;
-        t = t->first_child;
-    }
+    /* find the bottom node */
+    for (; t->first_child; t = t->first_child);
 
     /* when there are no modifiers in the binding, then the action cannot
        be interactive */
-    if (!mods && action->data.any.interactive) {
+    if (!t->state && action->data.any.interactive) {
+        g_print("not interactive\n");
         action->data.any.interactive = FALSE;
         action->data.inter.final = TRUE;
     }
@@ -206,6 +203,8 @@ static void keyboard_interactive_end(guint state, gboolean cancel, Time time,
     alist = g_slist_append(NULL, istate.action);
     action_run_interactive(alist, istate.client, state, time, cancel, TRUE);
     g_slist_free(alist);
+
+    keyboard_reset_chains(0);
 }
 
 static void keyboard_interactive_end_client(ObClient *client, gpointer data)
@@ -309,7 +308,14 @@ void keyboard_event(ObClient *client, const XEvent *e)
             } else if (p->chroot)         /* an empty chroot */
                 set_curpos(p);
             else {
-                keyboard_reset_chains(0);
+                GSList *it;
+                gboolean inter = FALSE;
+
+                for (it = p->actions; it && !inter; it = g_slist_next(it))
+                    if (((ObAction*)it->data)->data.any.interactive)
+                        inter = TRUE;
+                if (!inter) /* don't reset if the action is interactive */
+                    keyboard_reset_chains(0);
 
                 action_run_key(p->actions, client, e->xkey.state,
                                e->xkey.x_root, e->xkey.y_root,
