@@ -2,7 +2,7 @@
 
    mouse.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,8 @@
 #include "frame.h"
 #include "translate.h"
 #include "mouse.h"
+#include "gettext.h"
+
 #include <glib.h>
 
 typedef struct {
@@ -46,7 +48,8 @@ typedef struct {
 static GSList *bound_contexts[OB_FRAME_NUM_CONTEXTS];
 
 ObFrameContext mouse_button_frame_context(ObFrameContext context,
-                                          guint button)
+                                          guint button,
+                                          guint state)
 {
     GSList *it;
     ObFrameContext x = context;
@@ -54,7 +57,7 @@ ObFrameContext mouse_button_frame_context(ObFrameContext context,
     for (it = bound_contexts[context]; it; it = g_slist_next(it)) {
         ObMouseBinding *b = it->data;
 
-        if (b->button == button)
+        if (b->button == button && b->state == state)
             return context;
     }
 
@@ -63,16 +66,22 @@ ObFrameContext mouse_button_frame_context(ObFrameContext context,
     case OB_FRAME_CONTEXT_DESKTOP:
     case OB_FRAME_CONTEXT_CLIENT:
     case OB_FRAME_CONTEXT_TITLEBAR:
-    case OB_FRAME_CONTEXT_HANDLE:
     case OB_FRAME_CONTEXT_FRAME:
     case OB_FRAME_CONTEXT_MOVE_RESIZE:
+    case OB_FRAME_CONTEXT_LEFT:
+    case OB_FRAME_CONTEXT_RIGHT:
         break;
+    case OB_FRAME_CONTEXT_ROOT:
+        x = OB_FRAME_CONTEXT_DESKTOP;
+        break;
+    case OB_FRAME_CONTEXT_BOTTOM:
     case OB_FRAME_CONTEXT_BLCORNER:
     case OB_FRAME_CONTEXT_BRCORNER:
-        x = OB_FRAME_CONTEXT_HANDLE;
+        x = OB_FRAME_CONTEXT_BOTTOM;
         break;
     case OB_FRAME_CONTEXT_TLCORNER:
     case OB_FRAME_CONTEXT_TRCORNER:
+    case OB_FRAME_CONTEXT_TOP:
     case OB_FRAME_CONTEXT_MAXIMIZE:
     case OB_FRAME_CONTEXT_ALLDESKTOPS:
     case OB_FRAME_CONTEXT_SHADE:
@@ -85,7 +94,11 @@ ObFrameContext mouse_button_frame_context(ObFrameContext context,
         g_assert_not_reached();
     }
 
-    return x;
+    /* allow for multiple levels of fall-through */
+    if (x != context)
+        return mouse_button_frame_context(x, button, state);
+    else
+        return x;
 }
 
 void mouse_grab_for_client(ObClient *client, gboolean grab)
@@ -106,7 +119,7 @@ void mouse_grab_for_client(ObClient *client, gboolean grab)
                 mode = GrabModeAsync;
                 mask = ButtonPressMask | ButtonMotionMask | ButtonReleaseMask;
             } else if (CLIENT_CONTEXT(i, client)) {
-                win = client->frame->plate;
+                win = client->window;
                 mode = GrabModeSync; /* this is handled in event */
                 mask = ButtonPressMask; /* can't catch more than this with Sync
                                            mode the release event is
@@ -155,7 +168,7 @@ void mouse_unbind_all()
 
 static gboolean fire_binding(ObMouseAction a, ObFrameContext context,
                              ObClient *c, guint state,
-                             guint button, gint x, gint y)
+                             guint button, gint x, gint y, Time time)
 {
     GSList *it;
     ObMouseBinding *b;
@@ -168,7 +181,7 @@ static gboolean fire_binding(ObMouseAction a, ObFrameContext context,
     /* if not bound, then nothing to do! */
     if (it == NULL) return FALSE;
 
-    action_run_mouse(b->actions[a], c, context, state, button, x, y);
+    action_run_mouse(b->actions[a], c, context, state, button, x, y, time);
     return TRUE;
 }
 
@@ -177,7 +190,7 @@ void mouse_event(ObClient *client, XEvent *e)
     static Time ltime;
     static guint button = 0, state = 0, lbutton = 0;
     static Window lwindow = None;
-    static gint px, py;
+    static gint px, py, pwx = -1, pwy = -1;
 
     ObFrameContext context;
     gboolean click = FALSE;
@@ -185,29 +198,44 @@ void mouse_event(ObClient *client, XEvent *e)
 
     switch (e->type) {
     case ButtonPress:
-        context = frame_context(client, e->xany.window);
-        context = mouse_button_frame_context(context, e->xbutton.button);
+        context = frame_context(client, e->xbutton.window,
+                                e->xbutton.x, e->xbutton.y);
+        context = mouse_button_frame_context(context, e->xbutton.button,
+                                             e->xbutton.state);
 
         px = e->xbutton.x_root;
         py = e->xbutton.y_root;
+        if (!button) pwx = e->xbutton.x;
+        if (!button) pwy = e->xbutton.y;
         button = e->xbutton.button;
         state = e->xbutton.state;
 
         fire_binding(OB_MOUSE_ACTION_PRESS, context,
                      client, e->xbutton.state,
                      e->xbutton.button,
-                     e->xbutton.x_root, e->xbutton.y_root);
+                     e->xbutton.x_root, e->xbutton.y_root,
+                     e->xbutton.time);
+
+        /* if the bindings grab the pointer, there won't be a ButtonRelease
+           event for us */
+        if (grab_on_pointer())
+            button = 0;
 
         if (CLIENT_CONTEXT(context, client)) {
             /* Replay the event, so it goes to the client*/
-            XAllowEvents(ob_display, ReplayPointer, event_lasttime);
+            XAllowEvents(ob_display, ReplayPointer, event_curtime);
             /* Fall through to the release case! */
         } else
             break;
 
     case ButtonRelease:
-        context = frame_context(client, e->xany.window);
-        context = mouse_button_frame_context(context, e->xbutton.button);
+        /* use where the press occured in the window */
+        context = frame_context(client, e->xbutton.window, pwx, pwy);
+        context = mouse_button_frame_context(context, e->xbutton.button,
+                                             e->xbutton.state);
+
+        if (e->xbutton.button == button)
+            pwx = pwy = -1;
 
         if (e->xbutton.button == button) {
             /* clicks are only valid if its released over the window */
@@ -249,30 +277,32 @@ void mouse_event(ObClient *client, XEvent *e)
         fire_binding(OB_MOUSE_ACTION_RELEASE, context,
                      client, e->xbutton.state,
                      e->xbutton.button,
-                     e->xbutton.x_root, e->xbutton.y_root);
+                     e->xbutton.x_root,
+                     e->xbutton.y_root,
+                     e->xbutton.time);
         if (click)
             fire_binding(OB_MOUSE_ACTION_CLICK, context,
                          client, e->xbutton.state,
                          e->xbutton.button,
                          e->xbutton.x_root,
-                         e->xbutton.y_root);
+                         e->xbutton.y_root,
+                         e->xbutton.time);
         if (dclick)
             fire_binding(OB_MOUSE_ACTION_DOUBLE_CLICK, context,
                          client, e->xbutton.state,
                          e->xbutton.button,
                          e->xbutton.x_root,
-                         e->xbutton.y_root);
+                         e->xbutton.y_root,
+                         e->xbutton.time);
         break;
 
     case MotionNotify:
         if (button) {
-            context = frame_context(client, e->xany.window);
-            context = mouse_button_frame_context(context, button);
+            context = frame_context(client, e->xmotion.window, pwx, pwy);
+            context = mouse_button_frame_context(context, button, state);
 
-            if (ABS(e->xmotion.x_root - px) >=
-                config_mouse_threshold ||
-                ABS(e->xmotion.y_root - py) >=
-                config_mouse_threshold) {
+            if (ABS(e->xmotion.x_root - px) >= config_mouse_threshold ||
+                ABS(e->xmotion.y_root - py) >= config_mouse_threshold) {
 
                 /* You can't drag on buttons */
                 if (context == OB_FRAME_CONTEXT_MAXIMIZE ||
@@ -284,7 +314,7 @@ void mouse_event(ObClient *client, XEvent *e)
                     break;
 
                 fire_binding(OB_MOUSE_ACTION_MOTION, context,
-                             client, state, button, px, py);
+                             client, state, button, px, py, e->xmotion.time);
                 button = 0;
                 state = 0;
             }
@@ -305,13 +335,13 @@ gboolean mouse_bind(const gchar *buttonstr, const gchar *contextstr,
     GSList *it;
 
     if (!translate_button(buttonstr, &state, &button)) {
-        g_warning("invalid button '%s'", buttonstr);
+        g_message(_("Invalid button '%s' in mouse binding"), buttonstr);
         return FALSE;
     }
 
     context = frame_context_from_string(contextstr);
     if (!context) {
-        g_warning("invalid context '%s'", contextstr);
+        g_message(_("Invalid context '%s' in mouse binding"), contextstr);
         return FALSE;
     }
 
