@@ -2,7 +2,7 @@
 
    popup.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,21 +23,23 @@
 #include "frame.h"
 #include "client.h"
 #include "stacking.h"
+#include "event.h"
 #include "screen.h"
+#include "mainloop.h"
 #include "render/render.h"
 #include "render/theme.h"
 
-ObPopup *popup_new(gboolean hasicon)
+ObPopup *popup_new()
 {
     XSetWindowAttributes attrib;
     ObPopup *self = g_new0(ObPopup, 1);
 
     self->obwin.type = Window_Internal;
-    self->hasicon = hasicon;
     self->gravity = NorthWestGravity;
-    self->x = self->y = self->w = self->h = 0;
-    self->a_bg = RrAppearanceCopy(ob_rr_theme->app_hilite_bg);
-    self->a_text = RrAppearanceCopy(ob_rr_theme->app_hilite_label);
+    self->x = self->y = self->textw = self->h = 0;
+    self->a_bg = RrAppearanceCopy(ob_rr_theme->osd_hilite_bg);
+    self->a_text = RrAppearanceCopy(ob_rr_theme->osd_hilite_label);
+    self->iconwm = self->iconhm = 1;
 
     attrib.override_redirect = True;
     self->bg = XCreateWindow(ob_display, RootWindow(ob_display, ob_screen),
@@ -49,6 +51,10 @@ ObPopup *popup_new(gboolean hasicon)
                                0, 0, 1, 1, 0, RrDepth(ob_rr_inst),
                                InputOutput, RrVisual(ob_rr_inst), 0, NULL);
 
+    XSetWindowBorderWidth(ob_display, self->bg, ob_rr_theme->obwidth);
+    XSetWindowBorder(ob_display, self->bg,
+                     RrColorPixel(ob_rr_theme->osd_border_color));
+
     XMapWindow(ob_display, self->text);
 
     stacking_add(INTERNAL_AS_WINDOW(self));
@@ -74,25 +80,54 @@ void popup_position(ObPopup *self, gint gravity, gint x, gint y)
     self->y = y;
 }
 
-void popup_size(ObPopup *self, gint w, gint h)
+void popup_text_width(ObPopup *self, gint w)
 {
-    self->w = w;
-    self->h = h;
+    self->textw = w;
 }
 
-void popup_size_to_string(ObPopup *self, gchar *text)
+void popup_min_width(ObPopup *self, gint minw)
 {
-    gint textw, texth;
-    gint iconw;
+    self->minw = minw;
+}
 
-    self->a_text->texture[0].data.text.string = text;
-    RrMinsize(self->a_text, &textw, &texth);
-    /*XXX textw += ob_rr_theme->bevel * 2;*/
-    texth += ob_rr_theme->padding * 2;
+void popup_max_width(ObPopup *self, gint maxw)
+{
+    self->maxw = maxw;
+}
 
-    self->h = texth + ob_rr_theme->padding * 2;
-    iconw = (self->hasicon ? texth : 0);
-    self->w = textw + iconw + ob_rr_theme->padding * (self->hasicon ? 3 : 2);
+void popup_height(ObPopup *self, gint h)
+{
+    gint texth;
+
+    /* don't let the height be smaller than the text */
+    texth = RrMinHeight(self->a_text) + ob_rr_theme->paddingy * 2;
+    self->h = MAX(h, texth);
+}
+
+void popup_text_width_to_string(ObPopup *self, gchar *text)
+{
+    if (text[0] != '\0') {
+        self->a_text->texture[0].data.text.string = text;
+        self->textw = RrMinWidth(self->a_text);
+    } else
+        self->textw = 0;    
+}
+
+void popup_height_to_string(ObPopup *self, gchar *text)
+{
+    self->h = RrMinHeight(self->a_text) + ob_rr_theme->paddingy * 2;
+}
+
+void popup_text_width_to_strings(ObPopup *self, gchar **strings, gint num)
+{
+    gint i, maxw;
+
+    maxw = 0;
+    for (i = 0; i < num; ++i) {
+        popup_text_width_to_string(self, strings[i]);
+        maxw = MAX(maxw, self->textw);
+    }
+    self->textw = maxw;
 }
 
 void popup_set_text_align(ObPopup *self, RrJustify align)
@@ -100,64 +135,101 @@ void popup_set_text_align(ObPopup *self, RrJustify align)
     self->a_text->texture[0].data.text.justify = align;
 }
 
-void popup_show(ObPopup *self, gchar *text)
+static gboolean popup_show_timeout(gpointer data)
+{
+    ObPopup *self = data;
+    
+    XMapWindow(ob_display, self->bg);
+    stacking_raise(INTERNAL_AS_WINDOW(self));
+    self->mapped = TRUE;
+    self->delay_mapped = FALSE;
+
+    return FALSE; /* don't repeat */
+}
+
+void popup_delay_show(ObPopup *self, gulong usec, gchar *text)
 {
     gint l, t, r, b;
     gint x, y, w, h;
-    gint textw, texth;
-    gint iconw;
-    Rect *area; /* won't go outside this */
+    gint emptyx, emptyy; /* empty space between elements */
+    gint textx, texty, textw, texth;
+    gint iconx, icony, iconw, iconh;
+    Rect *area;
 
-    area = screen_physical_area();          /* XXX this should work quite
-                                               good, someone with xinerama,
-                                               and different resolutions on
-                                               screens? */
+    area = screen_physical_area();
 
-    RrMargins(self->a_bg, &l, &t, &r, &b);
-
-    XSetWindowBorderWidth(ob_display, self->bg, ob_rr_theme->bwidth);
-    XSetWindowBorder(ob_display, self->bg, ob_rr_theme->b_color->pixel);
+    /* when there is no icon and the text is not parent relative, then 
+       fill the whole dialog with the text appearance, don't use the bg at all
+    */
+    if (self->hasicon || self->a_text->surface.grad == RR_SURFACE_PARENTREL)
+        RrMargins(self->a_bg, &l, &t, &r, &b);
+    else
+        l = t = r = b = 0;
 
     /* set up the textures */
     self->a_text->texture[0].data.text.string = text;
 
-    /* measure the shit out */
-    RrMinsize(self->a_text, &textw, &texth);
-    /*XXX textw += ob_rr_theme->padding * 2;*/
-    texth += ob_rr_theme->padding * 2;
+    /* measure the text out */
+    if (text[0] != '\0') {
+        RrMinSize(self->a_text, &textw, &texth);
+    } else {
+        textw = 0;
+        texth = RrMinHeight(self->a_text);
+    }
 
-    /* set the sizes up and reget the text sizes from the calculated
-       outer sizes */
-    if (self->h) {
-        h = self->h;
-        texth = h - (t+b + ob_rr_theme->padding * 2);
+    /* get the height, which is also used for the icon width */
+    emptyy = t + b + ob_rr_theme->paddingy * 2;
+    if (self->h)
+        texth = self->h - emptyy;
+    h = texth * self->iconhm + emptyy;
+
+    if (self->textw)
+        textw = self->textw;
+
+    iconx = textx = l + ob_rr_theme->paddingx;
+
+    emptyx = l + r + ob_rr_theme->paddingx * 2;
+    if (self->hasicon) {
+        iconw = texth * self->iconwm;
+        iconh = texth * self->iconhm;
+        textx += iconw + ob_rr_theme->paddingx;
+        if (textw)
+            emptyx += ob_rr_theme->paddingx; /* between the icon and text */
     } else
-        h = t+b + texth + ob_rr_theme->padding * 2;
-    iconw = (self->hasicon ? texth : 0);
-    if (self->w) {
-        w = self->w;
-        textw = w - (l+r + iconw + ob_rr_theme->padding *
-                     (self->hasicon ? 3 : 2));
-    } else
-        w = l+r + textw + iconw + ob_rr_theme->padding *
-            (self->hasicon ? 3 : 2);
+        iconw = 0;
+
+    texty = (h - texth - emptyy) / 2 + t + ob_rr_theme->paddingy;
+    icony = (h - iconh - emptyy) / 2 + t + ob_rr_theme->paddingy;
+
+    /* when there is no icon, then fill the whole dialog with the text
+       appearance
+    */
+    if (!self->hasicon)
+    {
+        textx = texty = 0;
+        texth += emptyy;
+        textw += emptyx;
+        emptyx = emptyy = 0;
+    }
+
+    w = textw + emptyx + iconw;
+    /* cap it at maxw/minw */
+    if (self->maxw) w = MIN(w, self->maxw);
+    if (self->minw) w = MAX(w, self->minw);
+    textw = w - emptyx - iconw;
+
     /* sanity checks to avoid crashes! */
     if (w < 1) w = 1;
     if (h < 1) h = 1;
-    if (textw < 1) textw = 1;
     if (texth < 1) texth = 1;
 
     /* set up the x coord */
     x = self->x;
     switch (self->gravity) {
-    case NorthGravity:
-    case CenterGravity:
-    case SouthGravity:
+    case NorthGravity: case CenterGravity: case SouthGravity:
         x -= w / 2;
         break;
-    case NorthEastGravity:
-    case EastGravity:
-    case SouthEastGravity:
+    case NorthEastGravity: case EastGravity: case SouthEastGravity:
         x -= w;
         break;
     }
@@ -165,55 +237,67 @@ void popup_show(ObPopup *self, gchar *text)
     /* set up the y coord */
     y = self->y;
     switch (self->gravity) {
-    case WestGravity:
-    case CenterGravity:
-    case EastGravity:
+    case WestGravity: case CenterGravity: case EastGravity:
         y -= h / 2;
         break;
-    case SouthWestGravity:
-    case SouthGravity:
-    case SouthEastGravity:
+    case SouthWestGravity: case SouthGravity: case SouthEastGravity:
         y -= h;
         break;
     }
 
     x=MAX(MIN(x, area->width-w),0);
     y=MAX(MIN(y, area->height-h),0);
-
+    
     /* set the windows/appearances up */
     XMoveResizeWindow(ob_display, self->bg, x, y, w, h);
+    /* when there is no icon and the text is not parent relative, then 
+       fill the whole dialog with the text appearance, don't use the bg at all
+    */
+    if (self->hasicon || self->a_text->surface.grad == RR_SURFACE_PARENTREL)
+        RrPaint(self->a_bg, self->bg, w, h);
 
-    self->a_text->surface.parent = self->a_bg;
-    self->a_text->surface.parentx = l + iconw +
-        ob_rr_theme->padding * (self->hasicon ? 2 : 1);
-    self->a_text->surface.parenty = t + ob_rr_theme->padding;
-    XMoveResizeWindow(ob_display, self->text,
-                      l + iconw + ob_rr_theme->padding *
-                      (self->hasicon ? 2 : 1),
-                      t + ob_rr_theme->padding, textw, texth);
-
-    RrPaint(self->a_bg, self->bg, w, h);
-    RrPaint(self->a_text, self->text, textw, texth);
-
-    if (self->hasicon) {
-        if (iconw < 1) iconw = 1; /* sanity check for crashes */
-        if (self->draw_icon)
-            self->draw_icon(l + ob_rr_theme->padding, t + ob_rr_theme->padding,
-                            iconw, texth, self->draw_icon_data);
+    if (textw) {
+        self->a_text->surface.parent = self->a_bg;
+        self->a_text->surface.parentx = textx;
+        self->a_text->surface.parenty = texty;
+        XMoveResizeWindow(ob_display, self->text, textx, texty, textw, texth);
+        RrPaint(self->a_text, self->text, textw, texth);
     }
 
+    if (self->hasicon)
+        self->draw_icon(iconx, icony, iconw, iconh, self->draw_icon_data);
+
+    /* do the actual showing */
     if (!self->mapped) {
-        XMapWindow(ob_display, self->bg);
-        stacking_raise(INTERNAL_AS_WINDOW(self), FALSE);
-        self->mapped = TRUE;
+        if (usec) {
+            /* don't kill previous show timers */
+            if (!self->delay_mapped) {
+                ob_main_loop_timeout_add(ob_main_loop, usec,
+                                         popup_show_timeout, self,
+                                         g_direct_equal, NULL);
+                self->delay_mapped = TRUE;
+            }
+        } else {
+            popup_show_timeout(self);
+        }
     }
 }
 
 void popup_hide(ObPopup *self)
 {
     if (self->mapped) {
+        gulong ignore_start;
+
+        /* kill enter events cause by this unmapping */
+        ignore_start = event_start_ignore_all_enters();
+
         XUnmapWindow(ob_display, self->bg);
         self->mapped = FALSE;
+
+        event_end_ignore_all_enters(ignore_start);
+    } else if (self->delay_mapped) {
+        ob_main_loop_timeout_remove(ob_main_loop, popup_show_timeout);
+        self->delay_mapped = FALSE;
     }
 }
 
@@ -241,6 +325,7 @@ ObIconPopup *icon_popup_new()
                                RrVisual(ob_rr_inst), 0, NULL);
     XMapWindow(ob_display, self->icon);
 
+    self->popup->hasicon = TRUE;
     self->popup->draw_icon = icon_popup_draw_icon;
     self->popup->draw_icon_data = self;
 
@@ -257,18 +342,26 @@ void icon_popup_free(ObIconPopup *self)
     }
 }
 
-void icon_popup_show(ObIconPopup *self,
-                     gchar *text, const ObClientIcon *icon)
+void icon_popup_delay_show(ObIconPopup *self, gulong usec,
+                           gchar *text, const ObClientIcon *icon)
 {
     if (icon) {
         self->a_icon->texture[0].type = RR_TEXTURE_RGBA;
         self->a_icon->texture[0].data.rgba.width = icon->width;
         self->a_icon->texture[0].data.rgba.height = icon->height;
+        self->a_icon->texture[0].data.rgba.alpha = 0xff;
         self->a_icon->texture[0].data.rgba.data = icon->data;
     } else
         self->a_icon->texture[0].type = RR_TEXTURE_NONE;
 
-    popup_show(self->popup, text);
+    popup_delay_show(self->popup, usec, text);
+}
+
+void icon_popup_icon_size_multiplier(ObIconPopup *self, guint wm, guint hm)
+{
+    /* cap them at 1 */
+    self->popup->iconwm = MAX(1, wm);
+    self->popup->iconhm = MAX(1, hm);
 }
 
 static void pager_popup_draw_icon(gint px, gint py, gint w, gint h,
@@ -281,21 +374,18 @@ static void pager_popup_draw_icon(gint px, gint py, gint w, gint h,
     guint vert_inc;
     guint r, c;
     gint eachw, eachh;
+    const guint cols = screen_desktop_layout.columns;
+    const guint rows = screen_desktop_layout.rows;
+    const gint linewidth = ob_rr_theme->obwidth;
 
-    eachw = (w - ob_rr_theme->bwidth -
-             (screen_desktop_layout.columns * ob_rr_theme->bwidth))
-        / screen_desktop_layout.columns;
-    eachh = (h - ob_rr_theme->bwidth -
-             (screen_desktop_layout.rows * ob_rr_theme->bwidth))
-        / screen_desktop_layout.rows;
+    eachw = (w - ((cols + 1) * linewidth)) / cols;
+    eachh = (h - ((rows + 1) * linewidth)) / rows;
     /* make them squares */
     eachw = eachh = MIN(eachw, eachh);
 
     /* center */
-    px += (w - (screen_desktop_layout.columns * (eachw + ob_rr_theme->bwidth) +
-                ob_rr_theme->bwidth)) / 2;
-    py += (h - (screen_desktop_layout.rows * (eachh + ob_rr_theme->bwidth) +
-                ob_rr_theme->bwidth)) / 2;
+    px += (w - (cols * (eachw + linewidth) + linewidth)) / 2;
+    py += (h - (rows * (eachh + linewidth) + linewidth)) / 2;
 
     if (eachw <= 0 || eachh <= 0)
         return;
@@ -306,22 +396,22 @@ static void pager_popup_draw_icon(gint px, gint py, gint w, gint h,
         case OB_CORNER_TOPLEFT:
             n = 0;
             horz_inc = 1;
-            vert_inc = screen_desktop_layout.columns;
+            vert_inc = cols;
             break;
         case OB_CORNER_TOPRIGHT:
-            n = screen_desktop_layout.columns - 1;
+            n = cols - 1;
             horz_inc = -1;
-            vert_inc = screen_desktop_layout.columns;
+            vert_inc = cols;
             break;
         case OB_CORNER_BOTTOMRIGHT:
-            n = screen_desktop_layout.rows * screen_desktop_layout.columns - 1;
+            n = rows * cols - 1;
             horz_inc = -1;
             vert_inc = -screen_desktop_layout.columns;
             break;
         case OB_CORNER_BOTTOMLEFT:
-            n = (screen_desktop_layout.rows - 1) * screen_desktop_layout.columns;
+            n = (rows - 1) * cols;
             horz_inc = 1;
-            vert_inc = -screen_desktop_layout.columns;
+            vert_inc = -cols;
             break;
         default:
             g_assert_not_reached();
@@ -331,22 +421,22 @@ static void pager_popup_draw_icon(gint px, gint py, gint w, gint h,
         switch (screen_desktop_layout.start_corner) {
         case OB_CORNER_TOPLEFT:
             n = 0;
-            horz_inc = screen_desktop_layout.rows;
+            horz_inc = rows;
             vert_inc = 1;
             break;
         case OB_CORNER_TOPRIGHT:
-            n = screen_desktop_layout.rows * (screen_desktop_layout.columns - 1);
-            horz_inc = -screen_desktop_layout.rows;
+            n = rows * (cols - 1);
+            horz_inc = -rows;
             vert_inc = 1;
             break;
         case OB_CORNER_BOTTOMRIGHT:
-            n = screen_desktop_layout.rows * screen_desktop_layout.columns - 1;
-            horz_inc = -screen_desktop_layout.rows;
+            n = rows * cols - 1;
+            horz_inc = -rows;
             vert_inc = -1;
             break;
         case OB_CORNER_BOTTOMLEFT:
-            n = screen_desktop_layout.rows - 1;
-            horz_inc = screen_desktop_layout.rows;
+            n = rows - 1;
+            horz_inc = rows;
             vert_inc = -1;
             break;
         default:
@@ -358,11 +448,9 @@ static void pager_popup_draw_icon(gint px, gint py, gint w, gint h,
     }
 
     rown = n;
-    for (r = 0, y = 0; r < screen_desktop_layout.rows;
-         ++r, y += eachh + ob_rr_theme->bwidth)
+    for (r = 0, y = 0; r < rows; ++r, y += eachh + linewidth)
     {
-        for (c = 0, x = 0; c < screen_desktop_layout.columns;
-             ++c, x += eachw + ob_rr_theme->bwidth)
+        for (c = 0, x = 0; c < cols; ++c, x += eachw + linewidth)
         {
             RrAppearance *a;
 
@@ -391,9 +479,10 @@ ObPagerPopup *pager_popup_new()
 
     self->desks = 0;
     self->wins = g_new(Window, self->desks);
-    self->hilight = RrAppearanceCopy(ob_rr_theme->app_hilite_fg);
-    self->unhilight = RrAppearanceCopy(ob_rr_theme->app_unhilite_fg);
+    self->hilight = RrAppearanceCopy(ob_rr_theme->osd_hilite_fg);
+    self->unhilight = RrAppearanceCopy(ob_rr_theme->osd_unhilite_fg);
 
+    self->popup->hasicon = TRUE;
     self->popup->draw_icon = pager_popup_draw_icon;
     self->popup->draw_icon_data = self;
 
@@ -415,7 +504,8 @@ void pager_popup_free(ObPagerPopup *self)
     }
 }
 
-void pager_popup_show(ObPagerPopup *self, gchar *text, guint desk)
+void pager_popup_delay_show(ObPagerPopup *self, gulong usec,
+                            gchar *text, guint desk)
 {
     guint i;
 
@@ -430,9 +520,10 @@ void pager_popup_show(ObPagerPopup *self, gchar *text, guint desk)
         for (i = self->desks; i < screen_num_desktops; ++i) {
             XSetWindowAttributes attr;
 
-            attr.border_pixel = RrColorPixel(ob_rr_theme->b_color);
+            attr.border_pixel = 
+                RrColorPixel(ob_rr_theme->osd_border_color);
             self->wins[i] = XCreateWindow(ob_display, self->popup->bg,
-                                          0, 0, 1, 1, ob_rr_theme->bwidth,
+                                          0, 0, 1, 1, ob_rr_theme->obwidth,
                                           RrDepth(ob_rr_inst), InputOutput,
                                           RrVisual(ob_rr_inst), CWBorderPixel,
                                           &attr);
@@ -442,5 +533,12 @@ void pager_popup_show(ObPagerPopup *self, gchar *text, guint desk)
     self->desks = screen_num_desktops;
     self->curdesk = desk;
 
-    popup_show(self->popup, text);
+    popup_delay_show(self->popup, usec, text);
+}
+
+void pager_popup_icon_size_multiplier(ObPagerPopup *self, guint wm, guint hm)
+{
+    /* cap them at 1 */
+    self->popup->iconwm = MAX(1, wm);
+    self->popup->iconhm = MAX(1, hm);
 }
