@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    resist.h for the Openbox window manager
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,9 +23,17 @@ struct _ObClient;
 
 #include <glib.h>
 
-void resist_move_windows(struct _ObClient *c, gint *x, gint *y);
-void resist_move_monitors(struct _ObClient *c, gint *x, gint *y);
-void resist_size_windows(struct _ObClient *c, gint *w, gint *h, ObCorner corn);
-void resist_size_monitors(struct _ObClient *c, gint *w, gint *h,ObCorner corn);
+/*! @x The client's x destination (in the client's coordinates, not the frame's
+    @y The client's y destination (in the client's coordinates, not the frame's
+*/
+void resist_move_windows(struct _ObClient *c, gint resist, gint *x, gint *y);
+/*! @x The client's x destination (in the client's coordinates, not the frame's
+    @y The client's y destination (in the client's coordinates, not the frame's
+*/
+void resist_move_monitors(struct _ObClient *c, gint resist, gint *x, gint *y);
+void resist_size_windows(struct _ObClient *c, gint resist, gint *w, gint *h,
+                         ObCorner corn);
+void resist_size_monitors(struct _ObClient *c, gint resist, gint *w, gint *h,
+                          ObCorner corn);
 
 #endif
