@@ -2,7 +2,7 @@
 
    xerror.c for the Openbox window manager
    Copyright (c) 2006        Mikael Magnusson
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 */
 
 #include "openbox.h"
+#include "gettext.h"
+#include "debug.h"
 #include <glib.h>
 #include <X11/Xlib.h>
 
@@ -26,23 +28,22 @@ gboolean xerror_occured = FALSE;
 
 gint xerror_handler(Display *d, XErrorEvent *e)
 {
-    xerror_occured = TRUE;
 #ifdef DEBUG
-    if (!xerror_ignore) {
-        gchar errtxt[128];
+    gchar errtxt[128];
 
-        /*if (e->error_code != BadWindow) */
-        {
-            XGetErrorText(d, e->error_code, errtxt, 127);
-            if (e->error_code == BadWindow)
-                /*g_warning("X Error: %s", errtxt)*/;
-            else
-                g_error("X Error: %s", errtxt);
-        }
-    }
+    XGetErrorText(d, e->error_code, errtxt, 127);
+    if (!xerror_ignore) {
+        if (e->error_code == BadWindow)
+            /*g_message(_("X Error: %s\n"), errtxt)*/;
+        else
+            g_error(_("X Error: %s"), errtxt);
+    } else
+        ob_debug("XError code %d '%s'\n", e->error_code, errtxt);
 #else
     (void)d; (void)e;
 #endif
+
+    xerror_occured = TRUE;
     return 0;
 }
 
