@@ -23,7 +23,7 @@
 #include "color.h"
 #include <glib.h>
 
-static void highlight(RrPixel32 *x, RrPixel32 *y, gboolean raised);
+static void highlight(RrSurface *s, RrPixel32 *x, RrPixel32 *y, gboolean raised);
 static void gradient_parentrelative(RrAppearance *a, gint w, gint h);
 static void gradient_solid(RrAppearance *l, gint w, gint h);
 static void gradient_splitvertical(RrAppearance *a, gint w, gint h);
@@ -110,29 +110,29 @@ void RrRender(RrAppearance *a, gint w, gint h)
     if (a->surface.relief != RR_RELIEF_FLAT) {
         if (a->surface.bevel == RR_BEVEL_1) {
             for (off = 1, x = 1; x < w - 1; ++x, off++)
-                highlight(data + off,
+                highlight(&a->surface, data + off,
                           data + off + (h-1) * w,
                           a->surface.relief==RR_RELIEF_RAISED);
             for (off = 0, x = 0; x < h; ++x, off++)
-                highlight(data + off * w,
+                highlight(&a->surface, data + off * w,
                           data + off * w + w - 1,
                           a->surface.relief==RR_RELIEF_RAISED);
         }
 
         if (a->surface.bevel == RR_BEVEL_2) {
             for (off = 2, x = 2; x < w - 2; ++x, off++)
-                highlight(data + off + w,
+                highlight(&a->surface, data + off + w,
                           data + off + (h-2) * w,
                           a->surface.relief==RR_RELIEF_RAISED);
             for (off = 1, x = 1; x < h-1; ++x, off++)
-                highlight(data + off * w + 1,
+                highlight(&a->surface, data + off * w + 1,
                           data + off * w + w - 2,
                           a->surface.relief==RR_RELIEF_RAISED);
         }
     }
 }
 
-static void highlight(RrPixel32 *x, RrPixel32 *y, gboolean raised)
+static void highlight(RrSurface *s, RrPixel32 *x, RrPixel32 *y, gboolean raised)
 {
     gint r, g, b;
 
@@ -144,12 +144,13 @@ static void highlight(RrPixel32 *x, RrPixel32 *y, gboolean raised)
         up = y;
         down = x;
     }
+
     r = (*up >> RrDefaultRedOffset) & 0xFF;
-    r += r >> 1;
+    r += (r * s->bevel_light_adjust) >> 8;
     g = (*up >> RrDefaultGreenOffset) & 0xFF;
-    g += g >> 1;
+    g += (g * s->bevel_light_adjust) >> 8;
     b = (*up >> RrDefaultBlueOffset) & 0xFF;
-    b += b >> 1;
+    b += (b * s->bevel_light_adjust) >> 8;
     if (r > 0xFF) r = 0xFF;
     if (g > 0xFF) g = 0xFF;
     if (b > 0xFF) b = 0xFF;
@@ -157,11 +158,11 @@ static void highlight(RrPixel32 *x, RrPixel32 *y, gboolean raised)
         + (b << RrDefaultBlueOffset);
   
     r = (*down >> RrDefaultRedOffset) & 0xFF;
-    r = (r >> 1) + (r >> 2);
+    r -= (r * s->bevel_dark_adjust) >> 8;
     g = (*down >> RrDefaultGreenOffset) & 0xFF;
-    g = (g >> 1) + (g >> 2);
+    g -= (g * s->bevel_dark_adjust) >> 8;
     b = (*down >> RrDefaultBlueOffset) & 0xFF;
-    b = (b >> 1) + (b >> 2);
+    b -= (b * s->bevel_dark_adjust) >> 8;
     *down = (r << RrDefaultRedOffset) + (g << RrDefaultGreenOffset)
         + (b << RrDefaultBlueOffset);
 }
@@ -172,11 +173,11 @@ static void create_bevel_colors(RrAppearance *l)
 
     /* light color */
     r = l->surface.primary->r;
-    r += r >> 1;
+    r += (r * l->surface.bevel_light_adjust) >> 8;
     g = l->surface.primary->g;
-    g += g >> 1;
+    g += (g * l->surface.bevel_light_adjust) >> 8;
     b = l->surface.primary->b;
-    b += b >> 1;
+    b += (b * l->surface.bevel_light_adjust) >> 8;
     if (r > 0xFF) r = 0xFF;
     if (g > 0xFF) g = 0xFF;
     if (b > 0xFF) b = 0xFF;
@@ -185,11 +186,11 @@ static void create_bevel_colors(RrAppearance *l)
 
     /* dark color */
     r = l->surface.primary->r;
-    r = (r >> 1) + (r >> 2);
+    r -= (r * l->surface.bevel_dark_adjust) >> 8;
     g = l->surface.primary->g;
-    g = (g >> 1) + (g >> 2);
+    g -= (g * l->surface.bevel_dark_adjust) >> 8;
     b = l->surface.primary->b;
-    b = (b >> 1) + (b >> 2);
+    b -= (b * l->surface.bevel_dark_adjust) >> 8;
     g_assert(!l->surface.bevel_dark);
     l->surface.bevel_dark = RrColorNew(l->inst, r, g, b);
 }
@@ -420,13 +421,15 @@ static void gradient_solid(RrAppearance *l, gint w, gint h)
 
 static void gradient_splitvertical(RrAppearance *a, gint w, gint h)
 {
-    gint x, y1, y3, r, g, b;
+    gint x, y1, y2, y3, r, g, b;
     RrSurface *sf = &a->surface;
     RrPixel32 *data = sf->pixel_data;
     RrPixel32 current;
     RrColor *primary_light, *secondary_light;
+    gint y1sz, y2sz, y3sz;
 
     VARS(y1);
+    VARS(y2);
     VARS(y3);
 
     r = sf->primary->r;
@@ -438,7 +441,7 @@ static void gradient_splitvertical(RrAppearance *a, gint w, gint h)
     if (r > 0xFF) r = 0xFF;
     if (g > 0xFF) g = 0xFF;
     if (b > 0xFF) b = 0xFF;
-      primary_light = RrColorNew(a->inst, r, g, b);
+    primary_light = RrColorNew(a->inst, r, g, b);
 
     r = sf->secondary->r;
     r += r >> 4;
@@ -451,19 +454,35 @@ static void gradient_splitvertical(RrAppearance *a, gint w, gint h)
     if (b > 0xFF) b = 0xFF;
     secondary_light = RrColorNew(a->inst, r, g, b);
 
-    SETUP(y1, primary_light, sf->primary, (h / 2) -1);
-    SETUP(y3, sf->secondary, secondary_light,  (h / 2) -1);
+    y1sz = MAX(h/2 - 1, 1);
+    /* setup to get the colors _in between_ these other 2 */
+    y2sz = (h < 3 ? 0 : (h % 2 ? 3 : 2));
+    y3sz = MAX(h/2 - 1, 0);
 
-    for (y1 = h - 1; y1 > (h / 2) -1; --y1) {  /* 0 -> h-1 */
+    SETUP(y1, primary_light, sf->primary, y1sz);
+    if (y2sz) {
+        SETUP(y2, sf->primary, sf->secondary, y2sz);
+        NEXT(y2); /* skip the first one, its the same as the last of y1 */
+    }
+    SETUP(y3, sf->secondary, secondary_light,  y3sz);
+
+    for (y1 = y1sz; y1 > 0; --y1) {
         current = COLOR(y1);
-        for (x = w - 1; x >= 0; --x)  /* 0 -> w */
+        for (x = w - 1; x >= 0; --x)
             *(data++) = current;
 
         NEXT(y1);
     }
 
+    for (y2 = y2sz; y2 > 0; --y2) {
+        current = COLOR(y2);
+        for (x = w - 1; x >= 0; --x)
+            *(data++) = current;
+        
+        NEXT(y2);
+    }
     
-    for (y3 = (h / 2) - 1; y3 > 0; --y3) {
+    for (y3 = y3sz; y3 > 0; --y3) {
         current = COLOR(y3);
         for (x = w - 1; x >= 0; --x)
             *(data++) = current;
@@ -471,10 +490,6 @@ static void gradient_splitvertical(RrAppearance *a, gint w, gint h)
         NEXT(y3);
     }
 
-    current = COLOR(y3);
-    for (x = w - 1; x >= 0; --x)  /* 0 -> w */
-        *(data++) = current;
-
     RrColorFree(primary_light);
     RrColorFree(secondary_light);
 }
