@@ -123,6 +123,8 @@ struct _RrSurface {
     gint parentx;
     gint parenty;
     RrPixel32 *pixel_data;
+    gint bevel_dark_adjust;  /* 0-255, default is 64 */
+    gint bevel_light_adjust; /* 0-255, default is 128 */
 };
 
 struct _RrTextureText {
